###############################################################################
# BEGIN MPTL CONSTRAINTS
###############################################################################

# MPTL clock (250 MHz) (MGTREFCLK0_128)
#set_max_delay -from [get_ports {mptl_clk_?}]  100.0
#set_min_delay -from [get_ports {mptl_clk_?}] -100.0

create_clock -period 4.0 -name mptl_clk [get_ports {mptl_clk_p}]

# Sideband pins
#set_max_delay -to [get_ports {mptl_target_configured_n}]  100
#set_min_delay -to [get_ports {mptl_target_configured_n}] -100
#set_false_path -to [get_ports {mptl_target_configured_n}]

#set_max_delay -to [get_ports {mptl_target_gtp_online_n}]  100
#set_min_delay -to [get_ports {mptl_target_gtp_online_n}] -100
#set_false_path -to [get_ports {mptl_target_gtp_online_n}]

#set_max_delay -from [get_ports {mptl_bridge_gtp_online_n}]  100
#set_min_delay -from [get_ports {mptl_bridge_gtp_online_n}] -100
#set_false_path -from [get_ports {mptl_bridge_gtp_online_n}]


###############################################################################
# False Paths
###############################################################################

# done (GTH) and lock (GTH)
set_false_path        -through [get_pins        -hier -filter {NAME =~ *mptl128_interface_target_ku_1*gen_gthe3_channel_inst[?].GTHE3_CHANNEL_PRIM_INST/RXPMARESETDONE}]
set_false_path        -through [get_pins        -hier -filter {NAME =~ *mptl128_interface_target_ku_1*gen_gthe3_channel_inst[?].GTHE3_CHANNEL_PRIM_INST/CPLLLOCK}]
set_false_path        -through [get_pins        -hier -filter {NAME =~ *mptl128_interface_target_ku_1*gen_gthe3_channel_inst[?].GTHE3_CHANNEL_PRIM_INST/GTPOWERGOOD}]
set_false_path        -through [get_pins        -hier -filter {NAME =~ *mptl128_interface_target_ku_1*gen_gthe3_channel_inst[?].GTHE3_CHANNEL_PRIM_INST/RXCDRLOCK}]

set_false_path        -through [get_pins        -hier -filter {NAME =~ *mptl128_interface_target_ku_1*gen_gthe3_channel_inst[?].GTHE3_CHANNEL_PRIM_INST/TXPMARESETDONE}]
set_false_path        -through [get_pins        -hier -filter {NAME =~ *mptl128_interface_target_ku_1*gen_gthe3_channel_inst[?].GTHE3_CHANNEL_PRIM_INST/TXPRGDIVRESETDONE}]


###############################################################################
# Cross-clock domain constraints
# Define clock names
###############################################################################

# Clocks generated from primary clock GTH TXOUTCLK
#   mptl_clk_out 125 MHz clock (TXOUTCLK bufg_gt)

# GTHE3_CHANNEL_TXOUTCLK[0] (125 MHz)
set mptl_txoutclk [get_generated_clocks -of_objects [get_pins -hier -filter {NAME =~ *mptl128_interface_target_ku_1*gen_gthe3_channel_inst[3].GTHE3_CHANNEL_PRIM_INST/TXOUTCLK}]]
set mptl_clk_out ${mptl_txoutclk}

# Clocks generated from primary clock mptl_clk
#   mptl_mclk    250 MHz clock (IBUFGS_GTE3/O)
#   mptl_mclk_d2 125 MHz clock (IBUFGS_GTE3/ODIV2)

set mptl_mclk_d2 [get_generated_clocks -of_objects [get_pins -hier -filter {NAME =~ *mptl128_interface_target_ku_1*q3_clk0_refclk_ibufds_i/ODIV2}]]

# Clock coming in on aclk port, from user code
# We assume that a clock has been defined for aclk, somewhere in a user XDC file
set mptl_aclk [get_clocks -of_objects [get_ports {aclk}]]

###############################################################################
# Cross-clock domain constraints
# Define cross-clock delays = (source clock period) * (cc_period_factor)
###############################################################################

# cross-clock period factor
set cc_period_factor 0.8

set mptl_clk_out_delay [expr [get_property -min PERIOD $mptl_clk_out] * ${cc_period_factor}]
set mptl_mclk_d2_delay [expr [get_property -min PERIOD $mptl_mclk_d2] * ${cc_period_factor}]
set mptl_aclk_delay    [expr [get_property -min PERIOD $mptl_aclk]    * ${cc_period_factor}]


###############################################################################
# MPTL cross clock constraints
###############################################################################

# mptl_mclk_d2 (125 MHz) <=> mptl_clk_out (125 MHz)
set_max_delay -datapath_only -from ${mptl_mclk_d2} -to ${mptl_clk_out} ${mptl_mclk_d2_delay}
set_max_delay -datapath_only -from ${mptl_clk_out} -to ${mptl_mclk_d2} ${mptl_clk_out_delay}

# mptl_clk_out (125 MHz) <=> aclk
set_max_delay -datapath_only -from ${mptl_clk_out} -to ${mptl_aclk}    ${mptl_clk_out_delay}
set_max_delay -datapath_only -from ${mptl_aclk}    -to ${mptl_clk_out} ${mptl_aclk_delay}

# mptl_mclk_d2 (125 MHz) <=> aclk
set_max_delay -datapath_only -from ${mptl_mclk_d2} -to ${mptl_aclk}    ${mptl_mclk_d2_delay}
set_max_delay -datapath_only -from ${mptl_aclk}    -to ${mptl_mclk_d2} ${mptl_aclk_delay}

###############################################################################
# END MPTL CONSTRAINTS
###############################################################################
