#ifndef _ADATA_ADMXRC3UT_H
#define _ADATA_ADMXRC3UT_H

/*
** <admxrc3ut.h> - ADMXRC3 API Utility header file (for ADMXRC3 interface)
**
** (C) Copyright 2015 Alpha Data
**
** The integer datatypes used below that are prefixed by "_ADMXRC3UT_", such
** as "_ADMXRC3UT_UINT64" are defined in order to increase the portability of
** this header file but should NOT be used by application code that makes
** use of the ADMXRC3UT API.
**
** Applications should use OS-specific types such as UINT32 (Windows) or
** uint32_t (Linux C99).
*/

#include <admxrc3.h>

#if defined(_WIN32)

/* Windows */

typedef UINT32 _ADMXRC3UT_UINT32;
typedef BOOL   _ADMXRC3UT_BOOL;

#elif defined(__linux) || defined(__VXWORKS__) || defined(__vxworks)

/* Linux or VxWorks - assume C99-compliant */

# if defined(__linux)
#   include <stdint.h>
# endif

typedef uint32_t _ADMXRC3UT_UINT32;
typedef int      _ADMXRC3UT_BOOL;

#else

# warn Cannot detect target platform. Build will probably fail.

#endif

/*
** Function prototypes
*/
#ifdef __cplusplus
extern "C" {
#endif

/* Open a device by index, attempting to clear any device errors that may be present */
ADMXRC3_STATUS
ADMXRC3UT_OpenByIndex(
  unsigned int               index,
  _ADMXRC3UT_BOOL            bPassive,
  _ADMXRC3UT_BOOL            bClearErrors,
  _ADMXRC3UT_UINT32          cooperativeLevel,
  ADMXRC3_HANDLE*            phDevice);

/* Open a device by serial number, attempting to clear any device errors that may be present */
ADMXRC3_STATUS
ADMXRC3UT_OpenBySerialNumber(
  _ADMXRC3UT_UINT32          serialNumber,
  _ADMXRC3UT_BOOL            bPassive,
  _ADMXRC3UT_BOOL            bClearErrors,
  _ADMXRC3UT_UINT32          cooperativeLevel,
  ADMXRC3_HANDLE*            phDevice);

/* Return official name of product as UTF8 string */
const char*
ADMXRC3UT_GetModelNameA(
  ADMXRC3_MODEL_TYPE         model);
  
/* Return official name of product as UTF16 string */
const wchar_t*
ADMXRC3UT_GetModelNameW(
  ADMXRC3_MODEL_TYPE         model);

/* Return official name of product as UTF8/UTF16 string */
# ifdef _UNICODE
#   define ADMXRC3UT_GetModelName ADMXRC3UT_GetModelNameW
# else
#   define ADMXRC3UT_GetModelName ADMXRC3UT_GetModelNameA
# endif

/* Return name of product as UTF8 string that can be used when constructing a .bit filename */
const char*
ADMXRC3UT_GetModelTagA(
  ADMXRC3_MODEL_TYPE         model);
  
/* Return name of product as UTF16 string that can be used when constructing a .bit filename */
const wchar_t*
ADMXRC3UT_GetModelTagW(
  ADMXRC3_MODEL_TYPE         model);

/* Return name of product as UTF8/UTF16 string that can be used when constructing a .bit filename */
# ifdef _UNICODE
#   define ADMXRC3UT_GetModelIdentifier ADMXRC3UT_GetModelIdentifierW
# else
#   define ADMXRC3UT_GetModelIdentifier ADMXRC3UT_GetModelIdentifierA
# endif

#ifdef __cplusplus
}
#endif

#endif
